@@ -11,7 +11,7 @@
 #   Craig Barratt  <cbarratt@users.sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2003-2009  Craig Barratt
+#   Copyright (C) 2003-2013  Craig Barratt
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #
 #========================================================================
 #
-# Version 3.2.1, released 24 Apr 2011.
+# Version 3.3.0, released 14 Apr 2013.
 #
 # See http://backuppc.sourceforge.net.
 #
@@ -180,16 +180,33 @@ EOF
 
 sub timeStamp2
 {
+    my $now = $_[0] == 0 ? time : $_[0];
     my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
-              = localtime($_[0] == 0 ? time : $_[0] );
+              = localtime($now);
     $mon++;
     if ( $Conf{CgiDateFormatMMDD} == 2 ) {
         $year += 1900;
         return sprintf("%04d-%02d-%02d %02d:%02d", $year, $mon, $mday, $hour, $min);
     } elsif ( $Conf{CgiDateFormatMMDD} ) {
-        return sprintf("$mon/$mday %02d:%02d", $hour, $min);
+        #
+        # Add the year if the time is more than 330 days ago
+        #
+        if ( time - $now > 330 * 24 * 3600 ) {
+            $year -= 100;
+            return sprintf("$mon/$mday/%02d %02d:%02d", $year, $hour, $min);
+        } else {
+            return sprintf("$mon/$mday %02d:%02d", $hour, $min);
+        }
     } else {
-        return sprintf("$mday/$mon %02d:%02d", $hour, $min);
+        #
+        # Add the year if the time is more than 330 days ago
+        #
+        if ( time - $now > 330 * 24 * 3600 ) {
+            $year -= 100;
+            return sprintf("$mday/$mon/%02d %02d:%02d", $year, $hour, $min);
+        } else {
+            return sprintf("$mday/$mon %02d:%02d", $hour, $min);
+        }
     }
 }
 
